@@ -67,8 +67,6 @@ class Conn:
             _insert(dbconn, table_name, _rows2iter(val))
 
 
-# TODO: Not sure whether to check the syntax for each method.
-# for example, `fold` must be preceded by grouping methods (by, windowed, group)
 class Rows:
     """Contains a table source (database connection or list of dicts) and
     builds instructions as methods growing in chained
@@ -130,6 +128,8 @@ class Rows:
             slicing is similar to list slicing except that negative values
             cannot be used
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         if isinstance(key, str):
             return [r[key] for r in self._iter()]
         if isinstance(key, slice):
@@ -147,6 +147,10 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+        if isinstance(other, Rows):
+            _raise_exception_if_preceded_by_grouping_methods(other)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -171,6 +175,82 @@ class Rows:
 
         return newself
 
+    def merge(self, other, merge_type="inner"):
+        """Merges two Rows with zip_longest
+
+        If there are rows with the same column names,
+        'self' column values remain.
+
+        Parameters
+        ----------
+        other: Rows
+
+        join_type: str
+
+            'inner', 'left', 'right', 'full'
+            default: 'inner'
+
+        Returns
+        -------
+        Rows
+        """
+        _raise_exception_if_not_preceded_by_order_and_group(self)
+        _raise_exception_if_not_preceded_by_order_and_group(other)
+
+        merge_type = merge_type.lower()
+
+        def mergefn_new_row_ver(rows1, rows2, left, right_only):
+            if rows1 != [] and rows2 != []:
+                for row1, row2 in zip_longest(rows1, rows2):
+                    if row1 and row2:
+                        yield {**row1, **row2}
+                    elif row1 and not row2:
+                        yield {**row1, **{field: '' for field in right_only}}
+                    elif not row1 and row2:
+                        yield {**{field: '' for field in left}, **row2}
+
+            elif rows1 != [] and rows2 == []:
+                if merge_type in ("left", "full"):
+                    for row1 in rows1:
+                        yield {**row1, **{field: '' for field in right_only}}
+
+            elif rows1 == [] and rows2 != []:
+                if merge_type in ("right", "full"):
+                    for row2 in rows2:
+                        yield {**{field: '' for field in left}, **row2}
+
+        def mergefn_update_row_ver(rows1, rows2, left, right_only):
+            empty_left = {field: '' for field in left}
+            empty_right_only = {field: '' for field in right_only}
+
+            if rows1 != [] and rows2 != []:
+                for row1, row2 in zip_longest(rows1, rows2):
+                    if row1 and row2:
+                        row1.update(row2)
+                        yield row1
+                    elif row1 and not row2:
+                        row1.update(empty_right_only)
+                        yield row1
+                    elif not row1 and row2:
+                        empty_left.update(row2)
+                        yield empty_left
+
+            elif rows1 != [] and rows2 == []:
+                if merge_type in ("left", "full"):
+                    for row1 in rows1:
+                        row1.update(empty_right_only)
+                        yield row1
+
+            elif rows1 == [] and rows2 != []:
+                if merge_type in ("right", "full"):
+                    for row2 in rows2:
+                        empty_left.update(row2)
+                        yield empty_left
+
+        if self._history[0]['cmd'] == 'fetch':
+            return self._merge(mergefn_update_row_ver, other)
+        return self._merge(mergefn_new_row_ver, other)
+
     def zip_longest(self, other):
         """Attach two Rows side by side as long as any of them remained
 
@@ -184,48 +264,61 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+        if isinstance(other, Rows):
+            _raise_exception_if_preceded_by_grouping_methods(other)
 
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
 
-        def gen1():
+        def gen_new_row_ver():
             row1, rows1 = _spy(pgen())
             row2, rows2 = _spy(_rows2iter(other))
-            empty1 = {field: '' for field in row1.keys()}
-            empty2 = {field: '' for field in row2.keys()}
+
+            left = set(row1)
+            right_only = [field for field in list(row2) if field not in left]
+
+            empty_left = {field: '' for field in row1}
+            empty_right_only = {field: '' for field in right_only}
 
             for row1, row2 in zip_longest(rows1, rows2):
                 if row1 and row2:
                     yield {**row1, **row2}
                 elif row1 and not row2:
-                    yield {**row1, **empty2}
+                    yield {**row1, **empty_right_only}
                 elif not row1 and row2:
-                    yield {**empty1, **row2}
+                    yield {**empty_left, **row2}
 
-        def gen2():
+        def gen_update_row_ver():
             row1, rows1 = _spy(pgen())
             row2, rows2 = _spy(_rows2iter(other))
+
+            # fields
+            left = set(row1)
+            right_only = [field for field in list(row2) if field not in left]
+
             # using the same object over and over is SAFE
-            empty1 = {field: '' for field in row1.keys()}
-            empty2 = {field: '' for field in row2.keys()}
+            empty_left = {field: '' for field in row1}
+            empty_right_only = {field: '' for field in right_only}
 
             for row1, row2 in zip_longest(rows1, rows2):
                 if row1 and row2:
                     row1.update(row2)
                     yield row1
                 elif row1 and not row2:
-                    row1.update(empty2)
+                    row1.update(empty_right_only)
                     yield row1
                 elif not row1 and row2:
-                    empty1.update(row2)
-                    yield empty1
+                    empty_left.update(row2)
+                    yield empty_left
 
         # Do not use += operator here, it modifies the object
         # pylint: disable=protected-access
         newself._history = newself._history + [{
             'cmd': 'zip_longest',
-            'genfn': gen2 if self._history[0]['cmd'] == 'fetch' else gen1
+            'genfn': gen_update_row_ver if self._history[0]['cmd'] == 'fetch'
+            else gen_new_row_ver
         }]
         return newself
 
@@ -240,6 +333,10 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+        if isinstance(other, Rows):
+            _raise_exception_if_preceded_by_grouping_methods(other)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -273,8 +370,11 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         newself = copy(self)
 
+        # when fetching ordered table from the database.
         # pylint: disable=protected-access
         if len(newself._history) == 1\
                 and newself._history[-1]['cmd'] == 'fetch':
@@ -283,8 +383,9 @@ class Rows:
             # pylint: disable=protected-access
             newself._history = [{
                 **prehist,
+                'cmd_sub': 'order',
                 'genfn': _build_initgen(prehist['conn'], prehist['tname'],
-                                        fields_maybe_with_desc)
+                                        fields_maybe_with_desc),
             }]
             return newself
 
@@ -292,6 +393,7 @@ class Rows:
         pgen = newself._history[-1]['genfn']
 
         def gen_from_sql():
+            tmpdbfd, tmpdb = None, None
             try:
                 tmpdbfd, tmpdb = tempfile.mkstemp()
                 with _connect(tmpdb) as dbconn:
@@ -299,8 +401,10 @@ class Rows:
                     _insert(dbconn, temp_table_name, pgen())
                     yield from _fetch(dbconn, temp_table_name,
                                       fields_maybe_with_desc)
+            # Nothing to insert, from _insert
+            except ValueError:
+                yield from []
 
-            # TODO: possible errors must be dealt with.
             finally:
                 # must close the file descriptor to delete it
                 os.close(tmpdbfd)
@@ -382,6 +486,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -399,7 +505,6 @@ class Rows:
 
         return newself
 
-    # TODO: not sure 'join' can completely replace this one.
     def _merge(self, func, other):
         newself = copy(self)
         # pylint: disable=protected-access
@@ -487,6 +592,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         # need to cut out 'desc', 'asc'
         grouping_fields = [field.split()[0]
                            for field in fields_maybe_with_desc]
@@ -503,6 +610,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         def gen(row):
             if pred(row):
                 yield row
@@ -524,6 +633,7 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
 
         def updatefn1(row):
             newrow = row.copy()
@@ -558,6 +668,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         kwargs_rev = {oldkey: newkey for newkey, oldkey in kwargs.items()}
 
         def renamefn(row):
@@ -580,13 +692,15 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_not_preceded_by_grouping_methods(self)
+
         # fold must be preceded by `by`
         grouping_fields = []
         if self._history[-1]['cmd'] == 'group':
             grouping_fields = self._history[-1]['fields']
 
         # TODO: not sure whether to check if func returns a proper value
-        # (a str or a number)
+        # (str/int/float)
         def foldfn1(rows):
             rows = Rows(rows)
             row = {}
@@ -604,9 +718,8 @@ class Rows:
 
         if self._history[-1]['cmd'] == 'windowed':
             return self._apply(foldfn1)
-        if self._history[-1]['cmd'] == 'group':
-            return self._apply(foldfn2)
-        raise ValueError("fold must be preceded by grouping processes")
+        # then self._history[-1]['cmd'] must be 'group', it's check above
+        return self._apply(foldfn2)
 
     def select(self, *fields):
         """select fields discarding others
@@ -618,6 +731,8 @@ class Rows:
         Returns:
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         def selectfn(row):
             yield {field: row[field] for field in fields}
 
@@ -633,6 +748,8 @@ class Rows:
         Returns:
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         fields_set = set(fields)
 
         def deselectfn(row):
@@ -641,105 +758,37 @@ class Rows:
 
         return self._apply(deselectfn)
 
-    def join(self, other, join_type="left"):
+    def join(self, other, join_type="inner"):
         """Joins two Rows in SQL style
 
+        If there are rows with the same column names,
+        'self' column values are updated with 'other' column values.
+
         Parameters
         ----------
-        other: Rows/list[row]/Iterable[row]
+        other: Rows
 
         join_type: str
 
             'inner', 'left', 'right', 'full'
-            default: 'left'
-
+            default: 'inner'
         Returns
         -------
         Rows
         """
+        _raise_exception_if_not_preceded_by_order_and_group(self)
+        _raise_exception_if_not_preceded_by_order_and_group(other)
 
-        if join_type.lower() == "inner":
-            return self._inner_join(other)
-        if join_type.lower() == "left":
-            return self._left_join(other)
-        if join_type.lower() == "right":
-            return self._right_join(other)
-        if join_type.lower() == "full":
-            return self._full_join(other)
-        raise ValueError("Unknown join_type", join_type)
+        # Actually, if it's neither 'left', 'right', nor 'full',
+        # all the other strings are considered 'inner',
+        # however, this is not a feature.
+        # Specifically pass 'inner' if you want inner join
+        join_type = join_type.lower()
 
-    def _inner_join(self, other):
-        def innerjoin_fn1(rows1, rows2, _left, _right_only):
+        def joinfn_update_row_ver(rows1, rows2, left, right_only):
             if rows1 != [] and rows2 != []:
                 for row1, row2 in product(rows1, rows2):
-                    row1.update(row2)
-                    yield row1
-
-        def innerjoin_fn2(rows1, rows2, _left, _right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    yield {**row1, **row2}
-
-        if self._history[0]['cmd'] == 'fetch':
-            return self._merge(innerjoin_fn1, other)
-        return self._merge(innerjoin_fn2, other)
-
-    def _left_join(self, other):
-        def leftjoin_fn1(rows1, rows2, _left, right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    row1.update(row2)
-                    yield row1
-
-            elif rows1 != [] and rows2 == []:
-                for row1 in rows1:
-                    row1.update((field, '') for field in right_only)
-                    yield row1
-
-        def leftjoin_fn2(rows1, rows2, _left, right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    yield {**row1, **row2}
-
-            elif rows1 != [] and rows2 == []:
-                for row1 in rows1:
-                    yield {**row1, **{field: '' for field in right_only}}
-
-        if self._history[0]['cmd'] == 'fetch':
-            return self._merge(leftjoin_fn1, other)
-        return self._merge(leftjoin_fn2, other)
-
-    def _right_join(self, other):
-        def rightjoin_fn1(rows1, rows2, left, _right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    row1.update(row2)
-                    yield row1
-
-            elif rows1 == [] and rows2 != []:
-                empty_left = {field: '' for field in left}
-                for row2 in rows2:
-                    empty_left.update(row2)
-                    yield empty_left
-
-        def rightjoin_fn2(rows1, rows2, left, _right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    yield {**row1, **row2}
-
-            elif rows1 == [] and rows2 != []:
-                for row2 in rows2:
-                    yield {**{field: '' for field in left}, **row2}
-
-        if self._history[0]['cmd'] == 'fetch':
-            return self._merge(rightjoin_fn1, other)
-        return self._merge(rightjoin_fn2, other)
-
-    def _full_join(self, other):
-        def fulljoin_fn1(rows1, rows2, left, right_only):
-            if rows1 != [] and rows2 != []:
-                for row1, row2 in product(rows1, rows2):
-                    # TODO: updating on the same object multiple times.
+                    # updating on the same object multiple times.
                     # Still works, why ?!!,
                     # yielding and returning a appended list is different.
                     # later modifications do not affect previous yielded object
@@ -749,32 +798,36 @@ class Rows:
                     yield row1
 
             elif rows1 != [] and rows2 == []:
-                for row1 in rows1:
-                    row1.update((field, '') for field in right_only)
-                    yield row1
+                if join_type in ("left", "full"):
+                    for row1 in rows1:
+                        row1.update((field, '') for field in right_only)
+                        yield row1
 
             elif rows1 == [] and rows2 != []:
-                empty_left = {field: '' for field in left}
-                for row2 in rows2:
-                    empty_left.update(row2)
-                    yield empty_left
+                if join_type in ("right", "full"):
+                    empty_left = {field: '' for field in left}
+                    for row2 in rows2:
+                        empty_left.update(row2)
+                        yield empty_left
 
-        def fulljoin_fn2(rows1, rows2, left, right_only):
+        def joinfn_new_row_ver(rows1, rows2, left, right_only):
             if rows1 != [] and rows2 != []:
                 for row1, row2 in product(rows1, rows2):
                     yield {**row1, **row2}
 
             elif rows1 != [] and rows2 == []:
-                for row1 in rows1:
-                    yield {**row1, **{field: '' for field in right_only}}
+                if join_type in ("left", "full"):
+                    for row1 in rows1:
+                        yield {**row1, **{field: '' for field in right_only}}
 
             elif rows1 == [] and rows2 != []:
-                for row2 in rows2:
-                    yield {**{field: '' for field in left}, **row2}
+                if join_type in ("right", "full"):
+                    for row2 in rows2:
+                        yield {**{field: '' for field in left}, **row2}
 
         if self._history[0]['cmd'] == 'fetch':
-            return self._merge(fulljoin_fn1, other)
-        return self._merge(fulljoin_fn2, other)
+            return self._merge(joinfn_update_row_ver, other)
+        return self._merge(joinfn_new_row_ver, other)
 
     def distinct(self, *fields):
         """Returns a Rows with only the first row in each group.
@@ -790,6 +843,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         def distinctfn(_k, rows):
             # impossible to raise stop iteration here
             # pylint: disable=stop-iteration-return
@@ -816,6 +871,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -862,6 +919,8 @@ class Rows:
         Rows
 
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -889,6 +948,8 @@ class Rows:
         -------
         Rows
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         newself = copy(self)
         # pylint: disable=protected-access
         pgen = newself._history[-1]['genfn']
@@ -911,6 +972,7 @@ class Rows:
         -------
         int
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
 
         # There are rooms for this to be more efficient because some of the
         # methods do not affect the size of the Rows but
@@ -936,6 +998,8 @@ class Rows:
         -------
         list[row]
         """
+        _raise_exception_if_preceded_by_grouping_methods(self)
+
         # pylint: disable=protected-access
         return list(self._iter())
 
@@ -1013,7 +1077,7 @@ def _step(func, key_rows1, key_rows2):
 
         # all of left fields
         left = list(row1)
-        right_only = [field for field in list(row2) if field not in list(row1)]
+        right_only = [field for field in list(row2) if field not in set(row1)]
 
         while True:
             if key1 == key2:
@@ -1101,6 +1165,36 @@ def _windowed(seq, chunk_size, step):
         yield list(window)[i:]
 
 
+def _raise_exception_if_preceded_by_grouping_methods(rows_obj):
+    # pylint: disable=protected-access
+    cmd = rows_obj._history[-1]['cmd']
+    if cmd in ("group", "windowed"):
+        raise ValueError("Must not be preceded by grouping methods")
+
+
+def _raise_exception_if_not_preceded_by_grouping_methods(rows_obj):
+    # pylint: disable=protected-access
+    cmd = rows_obj._history[-1]['cmd']
+    if cmd not in ("group", "windowed"):
+        raise ValueError("Must be preceded by grouping methods")
+
+
+def _raise_exception_if_not_preceded_by_order_and_group(rows_obj):
+    # pylint: disable=protected-access
+    history = rows_obj._history
+    if len(history) < 2:
+        raise ValueError("Must be ordered and grouped before")
+
+    # when fetching ordered table from the database.
+    if len(history) == 2 and history[0].get('cmd_sub') == 'order':
+        return
+
+    cmd1 = history[-2]['cmd']
+    cmd2 = history[-1]['cmd']
+    if not (cmd1 == 'order' and cmd2 == 'group'):
+        raise ValueError("Must be ordered and grouped before")
+
+
 @contextmanager
 def _connect(dbfile):
     dbconn = sqlite3.connect(dbfile)
