@@ -36,6 +36,16 @@ class TestCore(unittest.TestCase):
         if Path(self.dbfile1).is_file():
             os.remove(self.dbfile1)
 
+    def test_order_when_no_row(self):
+        rs1 = Rows([])
+        self.assertEqual(rs1.order('col1').list(), [])
+
+        conn = Conn(self.dbfile)
+        conn['t1'] = self.t1
+        self.assertEqual(
+            conn['t1'].filter(lambda r: r['col2'] > 100).order('col1').list(),
+            [])
+
     def test_size(self):
         conn = Conn(self.dbfile)
         conn['t1'] = self.t1
@@ -598,7 +608,7 @@ class TestCore(unittest.TestCase):
     def test_join3(self):
         rs1 = Rows(self.t1)
         rs2 = Rows(self.t2)
-        rs = rs1.by('col1').join(rs2.by('col1'))
+        rs = rs1.by('col1').join(rs2.by('col1'), 'left')
         self.assertEqual(rs.list(),
                          [{'col1': 'a', 'col2': 4, 'col3': ''},
                           {'col1': 'a', 'col2': 5, 'col3': ''},
@@ -611,6 +621,97 @@ class TestCore(unittest.TestCase):
 
         self.assertEqual(rs1.list(), self.t1)
 
+    def test_join4(self):
+        rs1 = Rows(self.t1)
+        rs2 = Rows(self.t2)
+        rs1 = rs1.chain([{'col1': 'f', 'col2': 9},
+                         {'col1': 'f', 'col2': 8},
+                         {'col1': 'd', 'col2': 11},
+
+                         ])
+        rs1 = rs1.update(col4=lambda r: r['col2'] * 20)
+
+        rs2 = rs2.update(col2=lambda r: r['col3'] * 10)
+        rs3 = rs1.by('col1').join(rs2.by('col1'), 'full')
+        self.assertEqual(rs3.list(),
+                         [{'col1': 'a', 'col2': 4, 'col4': 80, 'col3': ''},
+                          {'col1': 'a', 'col2': 5, 'col4': 100, 'col3': ''},
+                          {'col1': 'b', 'col2': 10, 'col4': 20, 'col3': 1},
+                          {'col1': 'c', 'col2': 3, 'col4': 60, 'col3': ''},
+                          {'col1': 'c', 'col2': 4, 'col4': 80, 'col3': ''},
+                          {'col1': 'c', 'col2': 7, 'col4': 140, 'col3': ''},
+                          {'col1': 'd', 'col2': 30, 'col4': 40, 'col3': 3},
+                          {'col1': 'd', 'col2': 20, 'col4': 40, 'col3': 2},
+                          {'col1': 'd', 'col2': 30, 'col4': 220, 'col3': 3},
+                          {'col1': 'd', 'col2': 20, 'col4': 220, 'col3': 2},
+                          {'col1': 'e', 'col2': 30, 'col4': '', 'col3': 3},
+                          {'col1': 'e', 'col2': 40, 'col4': '', 'col3': 4},
+                          {'col1': 'f', 'col2': 9, 'col4': 180, 'col3': ''},
+                          {'col1': 'f', 'col2': 8, 'col4': 160, 'col3': ''},
+                          ])
+        conn = Conn(self.dbfile)
+        conn['t1'] = rs1
+        conn['t2'] = rs2
+        conn['t3'] = conn['t1'].by('col1').join(conn['t2'].by('col1'), 'full')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').join(rs2.by('col1'), 'left')
+        conn['t3'] = conn['t1'].by('col1').join(conn['t2'].by('col1'), 'left')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').join(rs2.by('col1'), 'right')
+        conn['t3'] = conn['t1'].by('col1').join(conn['t2'].by('col1'), 'right')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').join(rs2.by('col1'), 'inner')
+        conn['t3'] = conn['t1'].by('col1').join(conn['t2'].by('col1'), 'inner')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+    def test_merge(self):
+        rs1 = Rows(self.t1)
+        rs2 = Rows(self.t2)
+        rs1 = rs1.chain([{'col1': 'f', 'col2': 9},
+                         {'col1': 'f', 'col2': 8}])
+        rs2 = rs2.chain([{'col1': 'a', 'col3': 91}])
+
+        rs1 = rs1.update(col4=lambda r: r['col2'] * 20)
+        rs2 = rs2.update(col2=lambda r: r['col3'] * 10)
+
+        rs3 = rs1.by('col1').merge(rs2.by('col1'), 'full')
+        self.assertEqual(rs3.list(),
+                         [{'col1': 'a', 'col2': 910, 'col4': 80, 'col3': 91},
+                          {'col1': 'a', 'col2': 5, 'col4': 100, 'col3': ''},
+                          {'col1': 'b', 'col2': 10, 'col4': 20, 'col3': 1},
+                          {'col1': 'c', 'col2': 3, 'col4': 60, 'col3': ''},
+                          {'col1': 'c', 'col2': 4, 'col4': 80, 'col3': ''},
+                          {'col1': 'c', 'col2': 7, 'col4': 140, 'col3': ''},
+                          {'col1': 'd', 'col2': 30, 'col4': 40, 'col3': 3},
+                          {'col1': 'd', 'col2': 20, 'col4': '', 'col3': 2},
+                          {'col1': 'e', 'col2': 30, 'col4': '', 'col3': 3},
+                          {'col1': 'e', 'col2': 40, 'col4': '', 'col3': 4},
+                          {'col1': 'f', 'col2': 9, 'col4': 180, 'col3': ''},
+                          {'col1': 'f', 'col2': 8, 'col4': 160, 'col3': ''}])
+
+        conn = Conn(self.dbfile)
+        conn['t1'] = rs1
+        conn['t2'] = rs2
+        conn['t3'] = conn['t1'].by('col1').merge(conn['t2'].by('col1'), 'full')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').merge(rs2.by('col1'), 'left')
+        conn['t3'] = conn['t1'].by('col1').merge(conn['t2'].by('col1'), 'left')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').merge(rs2.by('col1'), 'right')
+        conn['t3'] = conn['t1'].by('col1').merge(
+            conn['t2'].by('col1'), 'right')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
+        rs3 = rs1.by('col1').merge(rs2.by('col1'), 'inner')
+        conn['t3'] = conn['t1'].by('col1').merge(
+            conn['t2'].by('col1'), 'inner')
+        self.assertEqual(rs3.list(), conn['t3'].list())
+
     def test_distinct(self):
         conn = Conn(self.dbfile)
         conn['t1'] = self.t1
